#include "assetmanager.h"

#include <fstream>
#include <sstream>
#include <json.hpp>
#include <assimp/Importer.hpp>
#include <assimp/scene.h>
#include <assimp/postprocess.h>
#include <opus.h>

#include "mesh.h"
#include "material.h"
#include "renderer.h"
#include "stringutils.h"
#include "audio.h"
#include "audiosystem.h"

MeshAsset* AssetManager::loadMesh(const std::string& path) {
    std::string fixedPath = "data/" + path;

    Assimp::Importer importer;
    const aiScene* scene = importer.ReadFile(fixedPath.c_str(), aiProcess_Triangulate);
    if(scene == nullptr)
        return nullptr;

    MeshAsset* mesh = new MeshAsset();

    unsigned int indexOffset = 0;
    for(unsigned mi = 0; mi < scene->mNumMeshes; mi++) {
        aiMesh* m = scene->mMeshes[mi];

        for(unsigned int i = 0; i < m->mNumVertices; i++) {
            Vertex vertex;
            vertex.position = glm::vec3(m->mVertices[i].x, m->mVertices[i].y, m->mVertices[i].z);
            vertex.normal = glm::vec3(m->mNormals[i].x, m->mNormals[i].y, m->mNormals[i].z);
            vertex.uv = glm::vec2(m->mTextureCoords[0][i].x, m->mTextureCoords[0][i].y);

            mesh->vertices.push_back(vertex);
        }

        for(unsigned int i = 0; i < m->mNumFaces; i++) {
            aiFace face = m->mFaces[i];
            for(unsigned int j = 0; j < face.mNumIndices; j++)
                mesh->indices.push_back(indexOffset + face.mIndices[j]);
        }

        indexOffset += m->mNumVertices;
    }

    renderer->fillMeshBuffers(mesh);

    return mesh;
}

MaterialAsset* AssetManager::loadMaterial(const std::string& path) {
    std::string fixedPath;
    if(path[0] != '/') // then this is an absolute path, and we absolutely do not want to mess with that
        fixedPath = "data/" + path;
    else
        fixedPath = path;

    std::ifstream file(fixedPath);
    if(!file)
        return nullptr;

    nlohmann::json json;
    file >> json;

    MaterialAsset* material = new MaterialAsset();

    auto tokens = tokenize(json["color"]);

    material->color[0] = atof(tokens[0].c_str());
    material->color[1] = atof(tokens[1].c_str());
    material->color[2] = atof(tokens[2].c_str());

    if(json["albedoTexture"].get<std::string>().length() > 0)
        material->albedoTexturePath = "data/" + json["albedoTexture"].get<std::string>();

    renderer->fillMaterialBuffers(material);

    return material;
}

AudioAsset* AssetManager::loadAudio(const std::string& path) {
    std::string fixedPath = "data/" + path;
    
    AudioAsset* audio = new AudioAsset();
    
    /*OggOpusFile* file = op_open_file(fixedPath.c_str(), nullptr);
    if(!file)
        return nullptr;

    audio->numChannels = op_channel_count(file, -1);
    int pcmSize = op_pcm_total(file, -1);

    audio->data = new int16_t[pcmSize * audio->numChannels];

    int samplesRead = 0;
    while(samplesRead < pcmSize) {
        int ns = op_read(file, audio->data + samplesRead * audio->numChannels, pcmSize * audio->numChannels, 0);

        samplesRead += ns;
    }

    op_free(file);
    
    audio->size = samplesRead * audio->numChannels * 2;
    
    audioSystem->fillAudioBuffers(audio);*/
    
    return audio;
}

