#include "audiosystem.h"

#include <al.h>

#include "ecs.h"
#include "worldmanager.h"
#include "audio.h"

AudioSystem::AudioSystem() {
    device = alcOpenDevice(nullptr);
    context = alcCreateContext(device, 0);

    alcMakeContextCurrent(context);
}

void AudioSystem::update() {
    const auto listeners = ECS::getWorldComponents<ListenerComponent>(worldManager.getCurrentWorld());

    if(listeners.size() > 0) {
        const auto& [id, listener] = listeners[0];
        const auto& transform = ECS::getComponent<TransformComponent>(id);

        alListener3f(AL_POSITION, transform->position.x, transform->position.y, transform->position.z);
    }
    
    const auto& speakers = ECS::getWorldComponents<SpeakerComponent>(worldManager.getCurrentWorld());
    
    for(const auto& [id, speaker] : speakers) {
        if(speaker->source == 0) {
            alGenSources(1, &speaker->source);
            alSourcei(speaker->source, AL_BUFFER, speaker->audio->buffer);
            
            alSourcePlay(speaker->source);
        }
        
        const auto& transform = ECS::getComponent<TransformComponent>(id);
        
        alSource3f(speaker->source, AL_POSITION, transform->position.x, transform->position.y, transform->position.z);
    }    
}

void AudioSystem::fillAudioBuffers(AudioAsset* asset) {
    alGenBuffers(1, &asset->buffer);
    
    ALenum format;
    if(asset->numChannels == 1)
        format = AL_FORMAT_MONO16;
    else if(asset->numChannels == 2)
        format = AL_FORMAT_STEREO16;

    alBufferData(asset->buffer, format, asset->data, asset->size, 48000);
}
