#include "entityparser.h"

#include "assetmanager.h"
#include "stringutils.h"

EntityID parseEntity(const nlohmann::json& json, World* world) {
    EntityID entity = ECS::createEntity(world);

    InfoComponent* info = ECS::addComponent<InfoComponent>(entity);
    info->name = json["name"];

    for(auto componentObject : json["components"]) {
        const auto& componentType = componentObject["type"];

        if(componentType == "Transform") {
            TransformComponent* transform = ECS::addComponent<TransformComponent>(entity);

            auto tokens = tokenize(componentObject["position"]);

            transform->position[0] = atof(tokens[0].c_str());
            transform->position[1] = atof(tokens[1].c_str());
            transform->position[2] = atof(tokens[2].c_str());
        } else if(componentType == "Mesh") {
            MeshComponent* mesh = ECS::addComponent<MeshComponent>(entity);

            mesh->mesh = assetManager.loadMesh(componentObject["path"]);
            mesh->material = assetManager.loadMaterial(componentObject["material"]);
        } else if(componentType == "Light") {
            LightComponent* light = ECS::addComponent<LightComponent>(entity);

            auto tokens = tokenize(componentObject["color"]);

            light->color[0] = atof(tokens[0].c_str());
            light->color[1] = atof(tokens[1].c_str());
            light->color[2] = atof(tokens[2].c_str());

            light->type = componentObject["kind"];
        } else if(componentType == "Camera") {
            ECS::addComponent<CameraComponent>(entity);
        } else if(componentType == "Listener") {
            ECS::addComponent<ListenerComponent>(entity);
        } else if(componentType == "Speaker") {
            auto listener = ECS::addComponent<SpeakerComponent>(entity);
            listener->audio = assetManager.loadAudio(componentObject["file"]);
        }
    }

    return entity;
}
