#include "worldmanager.h"

#include <fstream>
#include <sstream>
#include <iostream>
#include <json.hpp>

#include "stringutils.h"
#include "world.h"
#include "assetmanager.h"
#include "mesh.h"
#include "entityparser.h"
#include "ecs.h"
#include "renderer.h"
#include "mesh.h"
#include "material.h"

void WorldManager::loadWorld(const std::string& path) {
    std::ifstream file("data/" + path);
    if(!file)
        return;

    nlohmann::json json;
    file >> json;

    World* world = new World();

    auto tokens = tokenize(json["environment"]["color"]);

    world->color[0] = atof(tokens[0].c_str());
    world->color[1] = atof(tokens[1].c_str());
    world->color[2] = atof(tokens[2].c_str());

    for(const auto& entityObject : json["entities"])
        parseEntity(entityObject, world);

    loadedWorlds[path] = world;
}

void WorldManager::switchWorld(const std::string& path) {
    currentWorld = loadedWorlds[path];
}

void WorldManager::destructResources(Renderer* renderer) {
    auto meshes = ECS::getWorldComponents<MeshComponent>(currentWorld);
    for(const auto& [id, mesh] : meshes) {
        renderer->destroyMeshBuffers(mesh->mesh);
        renderer->destroyMaterialBuffers(mesh->material);

        delete mesh->mesh;
        delete mesh->material;
    }
}
